@@ -13,11 +13,12 @@ import secrets
 import struct
 import time
 from random import randint
-from typing import Any, Callable, Coroutine, Optional, Type, TypeVar
+from typing import Any, Callable, Coroutine, Optional, Type, TypeVar, final
 
 import aiohttp
 
 from .code_mappings import RoborockDockTypeCode
+from .command_cache import CacheableAttribute, CommandType, RoborockAttribute, create_cache_map, parse_method
 from .containers import (
     ChildLockStatus,
     CleanRecord,
@@ -54,9 +55,9 @@ from .exceptions import (
 )
 from .protocol import Utils
 from .roborock_future import RoborockFuture
-from .roborock_message import RoborockDataProtocol, RoborockMessage
+from .roborock_message import RoborockDataProtocol, RoborockMessage, RoborockMessageProtocol
 from .roborock_typing import DeviceProp, DockSummary, RoborockCommand
-from .util import fallback_cache, unpack_list
+from .util import RepeatableTask, get_running_loop_or_create_one, unpack_list
 
 _LOGGER = logging.getLogger(__name__)
 KEEPALIVE = 60
@@ -93,8 +94,42 @@ class PreparedRequest:
                 return await resp.json()
 
 
+EVICT_TIME = 60
+
+
+class AttributeCache:
+    def __init__(self, attribute: RoborockAttribute, api: RoborockClient):
+        self.attribute = attribute
+        self.api = api
+        self.attribute = attribute
+        self.task = RepeatableTask(self.api.event_loop, self._async_value, EVICT_TIME)
+        self._value: Any = None
+
+    @property
+    def value(self):
+        return self._value
+
+    async def _async_value(self):
+        self._value = await self.api._send_command(self.attribute.get_command)
+        return self._value
+
+    async def async_value(self):
+        if self._value is None:
+            return await self.task.reset()
+        return self._value
+
+    def stop(self):
+        self.task.cancel()
+
+    async def update_value(self, params):
+        response = await self.api._send_command(self.attribute.set_command, params)
+        await self._async_value()
+        return response
+
+
 class RoborockClient:
     def __init__(self, endpoint: str, device_info: DeviceData) -> None:
+        self.event_loop = get_running_loop_or_create_one()
         self.device_info = device_info
         self._endpoint = endpoint
         self._nonce = secrets.token_bytes(16)
@@ -103,9 +138,16 @@ class RoborockClient:
         self._last_disconnection = self.time_func()
         self.keep_alive = KEEPALIVE
         self._diagnostic_data: dict[str, dict[str, Any]] = {}
+        self.cache: dict[CacheableAttribute, AttributeCache] = {
+            cacheable_attribute: AttributeCache(attr, self) for cacheable_attribute, attr in create_cache_map().items()
+        }
 
     def __del__(self) -> None:
+        self.release()
+
+    def release(self):
         self.sync_disconnect()
+        [item.stop() for item in self.cache.values()]
 
     @property
     def diagnostic_data(self) -> dict:
@@ -134,7 +176,10 @@ class RoborockClient:
             self._last_device_msg_in = self.time_func()
             for data in messages:
                 protocol = data.protocol
-                if data.payload and (protocol == 102 or protocol == 4):
+                if data.payload and protocol in [
+                    RoborockMessageProtocol.RPC_RESPONSE,
+                    RoborockMessageProtocol.GENERAL_REQUEST,
+                ]:
                     payload = json.loads(data.payload.decode())
                     for data_point_number, data_point in payload.get("dps").items():
                         if data_point_number == "102":
@@ -158,7 +203,7 @@ class RoborockClient:
                                     if isinstance(result, list) and len(result) == 1:
                                         result = result[0]
                                     queue.resolve((result, None))
-                elif data.payload and protocol == 301:
+                elif data.payload and protocol == RoborockMessageProtocol.MAP_RESPONSE:
                     payload = data.payload[0:24]
                     [endpoint, _, request_id, _] = struct.unpack("<8s8sH6s", payload)
                     if endpoint.decode().startswith(self._endpoint):
@@ -238,30 +283,51 @@ class RoborockClient:
         )
         return request_id, timestamp, payload
 
+    async def send_message(self, roborock_message: RoborockMessage):
+        raise NotImplementedError
+
+    async def _send_command(
+        self,
+        method: RoborockCommand,
+        params: Optional[list | dict] = None,
+    ):
+        raise NotImplementedError
+
+    @final
     async def send_command(
         self,
         method: RoborockCommand,
         params: Optional[list | dict] = None,
         return_type: Optional[Type[RT]] = None,
     ) -> RT:
-        raise NotImplementedError
+        parsed_method = parse_method(method)
+        cache = None
+        if parsed_method is not None:
+            cache = self.cache[parsed_method.attribute]
+        response: Any = None
+        if cache is not None:
+            if parsed_method.type == CommandType.GET:
+                response = await cache.async_value()
+            elif parsed_method.type == CommandType.SET:
+                response = await cache.update_value(params)
+        else:
+            response = await self._send_command(method, params)
+        if return_type:
+            return return_type.from_dict(response)
+        return response
 
-    @fallback_cache
     async def get_status(self) -> Status | None:
         _cls: Type[Status] = ModelStatus.get(
             self.device_info.model, S7MaxVStatus
         )  # Default to S7 MAXV if we don't have the data
         return await self.send_command(RoborockCommand.GET_STATUS, return_type=_cls)
 
-    @fallback_cache
     async def get_dnd_timer(self) -> DnDTimer | None:
         return await self.send_command(RoborockCommand.GET_DND_TIMER, return_type=DnDTimer)
 
-    @fallback_cache
     async def get_valley_electricity_timer(self) -> ValleyElectricityTimer | None:
         return await self.send_command(RoborockCommand.GET_VALLEY_ELECTRICITY_TIMER, return_type=ValleyElectricityTimer)
 
-    @fallback_cache
     async def get_clean_summary(self) -> CleanSummary | None:
         clean_summary: dict | list | int = await self.send_command(RoborockCommand.GET_CLEAN_SUMMARY)
         if isinstance(clean_summary, dict):
@@ -278,27 +344,21 @@ class RoborockClient:
             return CleanSummary(clean_time=clean_summary)
         return None
 
-    @fallback_cache
     async def get_clean_record(self, record_id: int) -> CleanRecord | None:
         return await self.send_command(RoborockCommand.GET_CLEAN_RECORD, [record_id], return_type=CleanRecord)
 
-    @fallback_cache
     async def get_consumable(self) -> Consumable | None:
         return await self.send_command(RoborockCommand.GET_CONSUMABLE, return_type=Consumable)
 
-    @fallback_cache
     async def get_wash_towel_mode(self) -> WashTowelMode | None:
         return await self.send_command(RoborockCommand.GET_WASH_TOWEL_MODE, return_type=WashTowelMode)
 
-    @fallback_cache
     async def get_dust_collection_mode(self) -> DustCollectionMode | None:
         return await self.send_command(RoborockCommand.GET_DUST_COLLECTION_MODE, return_type=DustCollectionMode)
 
-    @fallback_cache
     async def get_smart_wash_params(self) -> SmartWashParams | None:
         return await self.send_command(RoborockCommand.GET_SMART_WASH_PARAMS, return_type=SmartWashParams)
 
-    @fallback_cache
     async def get_dock_summary(self, dock_type: RoborockDockTypeCode) -> DockSummary | None:
         """Gets the status summary from the dock with the methods available for a given dock.
 
@@ -320,16 +380,13 @@ class RoborockClient:
         )
         return DockSummary(dust_collection_mode, wash_towel_mode, smart_wash_params)
 
-    @fallback_cache
     async def get_prop(self) -> DeviceProp | None:
         """Gets device general properties."""
-        [status, clean_summary, consumable, dnd_timer, valley_electricity_timer] = await asyncio.gather(
+        [status, clean_summary, consumable] = await asyncio.gather(
             *[
                 self.get_status(),
                 self.get_clean_summary(),
                 self.get_consumable(),
-                self.get_dnd_timer(),
-                self.get_valley_electricity_timer(),
             ]
         )
         last_clean_record = None
@@ -338,27 +395,22 @@ class RoborockClient:
         dock_summary = None
         if status and status.dock_type is not None and status.dock_type != RoborockDockTypeCode.no_dock:
             dock_summary = await self.get_dock_summary(status.dock_type)
-        if any([status, dnd_timer, clean_summary, consumable]):
+        if any([status, clean_summary, consumable]):
             return DeviceProp(
                 status,
                 clean_summary,
                 consumable,
-                dnd_timer,
-                valley_electricity_timer,
                 last_clean_record,
                 dock_summary,
             )
         return None
 
-    @fallback_cache
     async def get_multi_maps_list(self) -> MultiMapsList | None:
         return await self.send_command(RoborockCommand.GET_MULTI_MAPS_LIST, return_type=MultiMapsList)
 
-    @fallback_cache
     async def get_networking(self) -> NetworkInfo | None:
         return await self.send_command(RoborockCommand.GET_NETWORK_INFO, return_type=NetworkInfo)
 
-    @fallback_cache
     async def get_room_mapping(self) -> list[RoomMapping] | None:
         """Gets the mapping from segment id -> iot id. Only works on local api."""
         mapping: list = await self.send_command(RoborockCommand.GET_ROOM_MAPPING)
@@ -369,17 +421,14 @@ class RoborockClient:
             ]
         return None
 
-    @fallback_cache
     async def get_child_lock_status(self) -> ChildLockStatus | None:
         """Gets current child lock status."""
         return await self.send_command(RoborockCommand.GET_CHILD_LOCK_STATUS, return_type=ChildLockStatus)
 
-    @fallback_cache
     async def get_flow_led_status(self) -> FlowLedStatus | None:
         """Gets current flow led status."""
         return await self.send_command(RoborockCommand.GET_FLOW_LED_STATUS, return_type=FlowLedStatus)
 
-    @fallback_cache
     async def get_sound_volume(self) -> int | None:
         """Gets current volume level."""
         return await self.send_command(RoborockCommand.GET_SOUND_VOLUME)
